@@ -5,11 +5,12 @@ files.
 """
 
 from __future__ import print_function, division
+import locale
+import io
 
 __developer_info__ = """
 Bruker file format information
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-
 Bruker binary files (ser/fid) store data as an array of numbers whose
 endianness is determined by the parameter BYTORDA (1 = big endian, 0 = little
 endian), and whose data type is determined by the parameter DTYPA (0 = int32,
@@ -121,7 +122,10 @@ def add_axis_to_udic(udic, dic, udim, strip_fake):
         pro_file = "procs"
 
     if acq_file in dic:
-        sw = dic[acq_file]["SW_h"]
+        if b_dim == 0:
+            sw = dic[acq_file]["SW_h"]
+        else:
+            sw = dic[acq_file]["SW"] * dic[acq_file]["SFO1"]
     elif pro_file in dic:
         sw = dic[pro_file]["SW_p"]
         # procNs files store sw (in Hz) with the 'SW_p' key instead of 'SW_h'.
@@ -807,7 +811,7 @@ def write_lowmem(dir, dic, data, bin_file=None, acqus_files=None,
     return
 
 
-def write_pdata(dir, dic, data, roll=False, shape=None, submatrix_shape=None, 
+def write_pdata(dir, dic, data, roll=False, shape=None, submatrix_shape=None,
                 scale_data=False, bin_file=None, procs_files=None,
                 write_procs=False, pdata_folder=False, overwrite=False,
                 big=None, isfloat=None, restrict_access=True):
@@ -835,7 +839,7 @@ def write_pdata(dir, dic, data, roll=False, shape=None, submatrix_shape=None,
         Apply a reverse scaling using the scaling factor defined in procs file
         By default, the array to be written will not be scaled using the value
         in procs but will be e scaled so  that the max intensity in that array
-        will have a value between 2**28 and 2**29. scale_data is to be used when 
+        will have a value between 2**28 and 2**29. scale_data is to be used when
         the array is itself a processed  bruker file that was read into nmrglue
     bin_file : str, optional
         Filename of binary file in directory. None uses standard files.
@@ -865,7 +869,7 @@ def write_pdata(dir, dic, data, roll=False, shape=None, submatrix_shape=None,
     # see that data consists of only real elements
     data = np.roll(data.real, int(roll))
 
-    # either apply a reverse scaling to the data or scale processed data 
+    # either apply a reverse scaling to the data or scale processed data
     # so that the max value is between 2**28 and 2**29 and cast to integers
     if scale_data:
         data = scale_pdata(dic, data, reverse=True)
@@ -2088,7 +2092,7 @@ def rm_dig_filter(
 
 # JCAMP-DX functions
 
-def read_jcamp(filename):
+def read_jcamp(filename, encoding=locale.getpreferredencoding()):
     """
     Read a Bruker JCAMP-DX file into a dictionary.
 
@@ -2100,6 +2104,8 @@ def read_jcamp(filename):
     ----------
     filename : str
         Filename of Bruker JCAMP-DX file.
+    encoding : str
+        Encoding of Bruker JCAMP-DX file. Defaults to the system default locale
 
     Returns
     -------
@@ -2118,7 +2124,7 @@ def read_jcamp(filename):
     """
     dic = {"_coreheader": [], "_comments": []}  # create empty dictionary
 
-    with open(filename, 'r') as f:
+    with io.open(filename, 'r', encoding=encoding) as f:
         while True:     # loop until end of file is found
 
             line = f.readline().rstrip()    # read a line
@@ -2194,13 +2200,21 @@ def parse_jcamp_value(text):
     """
     Parse value text from Bruker JCAMP-DX file returning the value.
     """
-    if "<" in text:
+    if text == '':
+        return None
+    elif text.startswith('<') and text.endswith('>'):
         return text[1:-1]  # remove < and >
-    elif "." in text or "e" in text or 'inf' in text:
-        return float(text)
     else:
-        return int(text)
-
+        if "." in text or "e" in text or 'inf' in text:
+            try:
+                return float(text)
+            except ValueError:
+                return text
+        else:
+            try:
+                return int(text)
+            except ValueError:
+                return text
 
 def write_jcamp(dic, filename, overwrite=False):
     """
