@@ -88,7 +88,7 @@ class converter(object):
         # processing for input type
         # sign alt. indirect dimension
         if data.ndim >= 2 and "alt_id_sign" in self._iproc:
-            #data[1::2] = -data[1::2]
+            # data[1::2] = -data[1::2]
             s = [slice(None, None, None)] * data.ndim
             for i in range(data.ndim - 1):
                 s[i] = slice(1, None, 2)
@@ -154,13 +154,13 @@ class converter(object):
         """
         # set data
         self._data = data
-        if udic != None and udic[0]['encoding'].lower() == "tppi":
+        if udic is not None and udic[0]['encoding'].lower() == "tppi":
             self._iproc = {"imagfactor": -1.0}
         else:   # states, etc needs sign alt. of indirect dim.
             self._iproc = {"alt_id_sign": True, "imagfactor": -1.0}
 
         # set the universal dictionary
-        if udic != None:
+        if udic is not None:
             self._udic = udic
         else:
             self._udic = varian.guess_udic(dic, data)
@@ -191,7 +191,7 @@ class converter(object):
             self._iproc = {}
 
         # set the universal dictionary
-        if udic != None:
+        if udic is not None:
             self._udic = udic
         else:
             self._udic = rnmrtk.guess_udic(dic, data)
@@ -215,7 +215,7 @@ class converter(object):
         self._iproc = {}
 
         # set the universal dictionary
-        if udic != None:
+        if udic is not None:
             self._udic = udic
         else:
             self._udic = pipe.guess_udic(dic, data)
@@ -239,12 +239,12 @@ class converter(object):
         self._iproc = {}
 
         # set the universal dictionary
-        if udic != None:
+        if udic is not None:
             self._udic = udic
         else:
             self._udic = sparky.guess_udic(dic, data)
 
-    def from_bruker(self, dic, data, udic=None):
+    def from_bruker(self, dic, data, udic=None, remove_digital_filter=False):
         """
         Load converter with Bruker data.
 
@@ -256,17 +256,26 @@ class converter(object):
             NMR data.
         udic : dict, optional
             Universal dictionary, if not provided will be guesses from dic.
+        remove_digital_filter : bool, optional
+            True to remove the Bruker digital filter.  Do not use this
+            option with low memory data or when the `udic` parameter is
+            specified.  False leave the digital filter in place.
+
 
         """
-        # set data
-        self._data = data
+        # set data, possibly with removal of filter delay
+        if remove_digital_filter:
+            self._data = bruker.remove_digital_filter(dic, data)
+        else:
+            self._data = data
+
         self._iproc = {}
 
         # set the universal dictionary
-        if udic != None:
+        if udic is not None:
             self._udic = udic
         else:
-            self._udic = bruker.guess_udic(dic, data)
+            self._udic = bruker.guess_udic(dic, self._data)
 
     # EXPORTERS (to_*)
     def to_universal(self):
